#ifndef DBF_H
#define DBF_H

#include <stdio.h>
#include "bool.h"

#ifndef uchar
#define uchar unsigned char
#endif

#ifndef ushort
#define ushort unsigned short
#endif

#ifndef ulong
#define ulong unsigned long
#endif

#define MAXPATH 127

#define F_BROWSE 0x01
#define F_FDESC  0x02
#define F_HDR    0x04
#define F_SQLERR 0x07
#define F_SQL    0x08
#define F_TRIM   0x10
#define F_RDB    0x20
#define F_NOSQL  0x40
#define F_APPEND 0x80
#define F_MYSQL  0x100
#define F_SQL_INSERT    0x0200
#define F_OMITDEL 0x0400

typedef struct
{
   char fbuf[32];
   char name[11];
   uchar type;
   uchar length;
   uchar decimals;
   unsigned long offset;

} dbf_fspec;

dbf_fspec *dbf_fspec_from_dbf(char[32], ulong);
void dbf_fspec_dump(dbf_fspec *);

typedef struct
{
	char memofilename[MAXPATH];
	FILE *memofp;
	unsigned short blocksize;
	unsigned int nextfree;

	int blocksig;
	unsigned int memolen;
	char *mbuf;

} dbf_memo;

void dbf_memo_init(dbf_memo *, char *);
void dbf_memo_free(dbf_memo *);
char *dbf_memo_get_memo(dbf_memo *, long);

typedef struct
{
   // main dbf header from file

   uchar version_id;
   char last_update[3];
   ulong rcount;
   ulong falloced;
   ushort hlength;
   ushort rlength;

   unsigned int fcount;
   char filename[MAXPATH];
   FILE *datafile;
   char *rbuf;
   char *hbuf;

   bool has_memo;
   dbf_memo memo;

   dbf_fspec **fields;


} dbf_table;

dbf_table *dbf_from_rdb(char *, char *);
dbf_table *dbf_open(char *);
void dbf_free(dbf_table *);
int dbf_read(dbf_table *, ulong);

// dump routines

void dbf_dump(dbf_table *, char, int);
void dbf_dump_header(dbf_table *);
void dbf_dump_fspecs(dbf_table *);
void dbf_dump_nosql(dbf_table *, char *, int);
void dbf_header2sql(dbf_table *);
void dbf_header2sql_insert(int, dbf_table *);
void dbf_data2sql(dbf_table *, char, int);
void dbf_data2sql_insert(dbf_table *, char, int);
void dbf_rdb_append(dbf_table *, char *);

#endif
